'use strict'
var axios = require('axios')
var fs = require('fs')
var P = require('bluebird')

var NOW_API_TOKEN = 'NOW_API_TOKEN'

/*
 * Parses a string or buffer into an object
 * @param {(string|Buffer)} src - source to be parsed
 * @returns {Object} keys and values from src
*/
function parse (src) {
  var obj = {}

  // convert Buffers before splitting into lines and processing
  src.toString().split('\n').forEach(function (line) {
    // matching "KEY' and 'VAL' in 'KEY=VAL'
    var keyValueArr = line.match(/^\s*([\w\.\-]+)\s*=\s*(.*)?\s*$/)
    // matched?
    if (keyValueArr != null) {
      var key = keyValueArr[1]

      // default undefined or missing values to empty string
      var value = keyValueArr[2] ? keyValueArr[2] : ''

      // expand newlines in quoted values
      var len = value ? value.length : 0
      if (len > 0 && value.charAt(0) === '"' && value.charAt(len - 1) === '"') {
        value = value.replace(/\\n/gm, '\n')
      }

      // remove any surrounding quotes and extra spaces
      value = value.replace(/(^['"]|['"]$)/g, '').trim()

      obj[key] = value
    }
  })

  return obj
}

/*
 * Main entry point into dotenv. Allows configuration before loading .env
 * @param {Object} options - options for parsing .env file
 * @param {string} [options.path=.env] - path to .env file
 * @param {string} [options.encoding=utf8] - encoding of .env file
 * @returns {Object} parsed object or error
*/
function config (options) {
  var path = '.envnow'
  var encoding = 'utf8'

  if (options) {
    if (options.path) {
      path = options.path
    }
    if (options.encoding) {
      encoding = options.encoding
    }
  }

  try {
    // specifying an encoding returns a string instead of a buffer
    var parsedObj = parse(fs.readFileSync(path, { encoding: encoding }))

    if (!parsedObj[NOW_API_TOKEN]) {
      throw new Error('No now API token provided')
    }
    return P.all(Object.keys(parsedObj).filter(key => {
      return key !== NOW_API_TOKEN
    }).map(key => {
      return axios({
        method: 'POST',
        url: 'https://api.zeit.co/now/secrets',
        headers: {
          'Authorization': `Bearer ${parsedObj[NOW_API_TOKEN]}`
        },
        data: {
          name: key,
          value: parsedObj[key]
        }
      })
        .then(result => {
          console.log(result.data)
        })
        .catch(error => {
          console.error(error.response.data.error)
        })
    }))

  } catch (e) {
    return { error: e }
  }
}

function get () {
  var path = '.envnow'
  var encoding = 'utf8'

  var parsedObj = parse(fs.readFileSync(path, { encoding: encoding }))

  if (!parsedObj[NOW_API_TOKEN]) {
    throw new Error('No now API token provided')
  }

  return axios({
    method: 'GET',
    url: 'https://api.zeit.co/now/secrets',
    headers: {
      'Authorization': `Bearer ${parsedObj[NOW_API_TOKEN]}`
    }
  })
    .then(result => {
      console.log(result.data)
    })
    .catch(error => {
      console.error(error.data)
    })

}

module.exports.get = get
module.exports.config = config
module.exports.load = config
module.exports.parse = parse
